//Training a binary classifier on a data set. This code trains a model of binary classifier by using a data set read from STDIN, and writes the resultant model to STDOUT. This code assumes that features in the data set are represented by integer identifiers.
/*
 *      A sample program for training a binary classifier.
 *
 * Copyright (c) 2008,2009 Naoaki Okazaki
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the names of the authors nor the names of its contributors
 *       may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: train_binary_batch.cpp 180 2009-09-25 08:51:06Z naoaki $ */

/*
 * This file was modified from train_binary_batch.cpp (info above) 
 * by Kevin Gimpel.
 * Copyright 2010-2012 Kevin Gimpel
 * Released under the modified BSD license. See the LICENSE file included with this distribution.
 */

#include "classias_binary_classifier.h"
#include <iostream>
#include <string>
#include "utils.h"

ClassiasBinaryClassifier::ClassiasBinaryClassifier() {
	;
}

void ClassiasBinaryClassifier::addTrainingExample(const map<int, double>& featureVector, bool label) {
	// Create a new instance in the data set.
	classias::binstance& inst = m_data.new_element();
	// Set the boolean label for the instance.
	inst.set_label(label);

	for (map<int, double>::const_iterator iter = featureVector.begin(); iter != featureVector.end(); ++iter) {
		string featureName = itos(iter->first);
		double featureValue = iter->second;
		// Append the feature to the instance.
		inst.append(m_data.attributes(featureName), featureValue);
	}
}

void ClassiasBinaryClassifier::train(int maxNumIterations, double c1, double c2) {
	cerr << "Training binary classifier." << endl;
	cerr << "m_data.num_features() = " << m_data.num_features() << endl;
	cerr << "m_data.size() = " << m_data.size() << endl;

    // Set some parameters for the training algorithm.
	m_lbfgsLogisticRegressionClassifier.params().set("max_iterations", maxNumIterations);
	m_lbfgsLogisticRegressionClassifier.params().set("c1", c1);
	m_lbfgsLogisticRegressionClassifier.params().set("c2", c2);

	// Start training; progress report will be shown in es (STDERR).
	m_lbfgsLogisticRegressionClassifier.train(m_data, cerr);

	// Print and save the model.
	//const classias::weight_vector& w = m_lbfgsLogisticRegressionClassifier.model();
	m_sparseWeightVector.clear();
	const classias::weight_vector& w = m_lbfgsLogisticRegressionClassifier.model();
    for (int i = 0; i < w.size(); ++i) {
        // Feature ID and its weight.
        int modelFeatureID = atoi((m_data.attributes.to_item(i)).c_str());
        cerr << i << "\t" << modelFeatureID << "\t" << w[i] << endl;
		m_sparseWeightVector[modelFeatureID] = w[i];
    }
}

